from ..constants import (CONST_FIELD_KEYWORD, CONST_FIELD_MATCH,
                         CONST_FIELD_MATCHEND, CONST_FIELD_TARGET)


class SubcorpusCollection:
    def __init__(self, client, corpus):
        self.client = client
        self.corpus = corpus

    def get(self, name):
        return Subcorpus(self.client, self.corpus, name)

    def list(self):
        return [Subcorpus(self.client, self.corpus, subcorpus) for subcorpus in
                self.client.api.cqp_list_subcorpora(self.corpus.name)]


class Subcorpus:
    def __init__(self, client, corpus, name):
        self.client = client
        self.corpus = corpus
        self.name = name
        self._name = '{}:{}'.format(corpus.name, name)
        self.fields = {}
        if client.api.cqp_subcorpus_has_field(self._name, CONST_FIELD_MATCH):
            self.fields['match'] = CONST_FIELD_MATCH
        if client.api.cqp_subcorpus_has_field(self._name,
                                              CONST_FIELD_MATCHEND):
            self.fields['matchend'] = CONST_FIELD_MATCHEND
        if client.api.cqp_subcorpus_has_field(self._name, CONST_FIELD_TARGET):
            self.fields['target'] = CONST_FIELD_TARGET
        if client.api.cqp_subcorpus_has_field(self._name, CONST_FIELD_KEYWORD):
            self.fields['keyword'] = CONST_FIELD_KEYWORD
        self.size = client.api.cqp_subcorpus_size(self._name)

    def drop(self):
        return self.client.api.cqp_drop_subcorpus(self._name)

    def dump(self, field, first, last):
        return self.client.api.cqp_dump_subcorpus(self._name, field, first,
                                                  last)

    def dump_values(self, context=25, first_result=0,
                    num_results=float('inf')):
        if self.size == 0:
            return {"matches": []}
        first_result = max(0, first_result)
        last_result = min((first_result + num_results - 1), (self.size - 1))
        matches = []
        match_boundaries = zip(self.dump(self.fields['match'], first_result,
                                         last_result),
                               self.dump(self.fields['matchend'], first_result,
                                         last_result))
        for match_start, match_end in match_boundaries:
            left_start = max(0, match_start - context)
            right_end = min(self.corpus.size, (match_end + 1 + context))
            matches.append({'lc': list(range(left_start, match_start)),
                            'hit': list(range(match_start, match_end + 1)),
                            'rc': list(range(match_end + 1, right_end))})
        cpos_list = []
        for match in matches:
            cpos_list += match['lc'] + match['hit'] + match['rc']
        lookups = self.corpus.lookups_by_cpos(cpos_list)
        return {'matches': matches, **lookups}

    def fdist_1(self, cutoff, field, attribute):
        return self.client.api.cqp_fdist_1(self._name, cutoff, field,
                                           attribute._name)

    def fdist_2(self, cutoff, field_1, attribute_1, field_2, attribute_2):
        return self.client.api.cqp_fdist_2(self._name, cutoff,
                                           field_1, attribute_1._name,
                                           field_2, attribute_2._name)
